@@ -1,5 +1,6 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:critical reason:authorization-protocol
 
 #include "qoauth1.h"
 #include "qoauth1_p.h"
@@ -150,8 +151,8 @@ QNetworkReply *QOAuth1Private::requestToken(QNetworkAccessManager::Operation ope
     QMultiMap<QString, QVariant> remainingParameters;
     appendCommonHeaders(&headers);
     for (auto it = parameters.begin(), end = parameters.end(); it != end; ++it) {
-        const auto key = it.key();
-        const auto value = it.value();
+        const auto &key = it.key();
+        const auto &value = it.value();
         if (key.startsWith(QStringLiteral("oauth_")))
             headers.insert(key, value);
         else
@@ -176,7 +177,7 @@ QNetworkReply *QOAuth1Private::requestToken(QNetworkAccessManager::Operation ope
     }
     else if (operation == QNetworkAccessManager::PostOperation) {
         QUrlQuery query = QOAuth1Private::createQuery(remainingParameters);
-        const QByteArray data = query.toString(QUrl::FullyEncoded).toUtf8();
+        const QByteArray data = query.toString(QUrl::FullyEncoded).toLatin1();
         request.setHeader(QNetworkRequest::ContentTypeHeader,
                           QStringLiteral("application/x-www-form-urlencoded"));
         reply = networkAccessManager()->post(request, data);
@@ -275,7 +276,8 @@ void QOAuth1::prepareRequest(QNetworkRequest *request, const QByteArray &verb,
         request->header(QNetworkRequest::ContentTypeHeader).toByteArray()
             == "application/x-www-form-urlencoded") {
         QUrlQuery query(QString::fromUtf8(body));
-        for (const auto &item : query.queryItems(QUrl::FullyDecoded))
+        const auto items = query.queryItems(QUrl::FullyDecoded);
+        for (const auto &item : items)
             signingParams.insert(item.first, item.second);
     }
     setup(request, signingParams, verb);
