@@ -1,9 +1,9 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+// Qt-Security score:critical reason:authorization-protocol
 
 #include "qoauthoobreplyhandler.h"
 #include "qoauthoobreplyhandler_p.h"
-#include "qabstractoauthreplyhandler_p.h"
 
 #include <QtCore/qurlquery.h>
 #include <QtCore/qjsonobject.h>
@@ -26,6 +26,9 @@ QOAuthOobReplyHandler::QOAuthOobReplyHandler(QOAuthOobReplyHandlerPrivate &d, QO
     : QAbstractOAuthReplyHandler(d, parent)
 {}
 
+QOAuthOobReplyHandler::~QOAuthOobReplyHandler()
+    = default; // must be empty until Qt 7 (was inline until Qt 6.8)
+
 QString QOAuthOobReplyHandler::callback() const
 {
     return QStringLiteral("oob");
@@ -92,7 +95,7 @@ QVariantMap QOAuthOobReplyHandler::parseResponse(const QByteArray &response)
 {
     QVariantMap ret;
     QUrlQuery query(QString::fromUtf8(response));
-    auto queryItems = query.queryItems(QUrl::FullyDecoded);
+    const auto queryItems = query.queryItems(QUrl::FullyDecoded);
     for (auto it = queryItems.begin(), end = queryItems.end(); it != end; ++it)
         ret.insert(it->first, it->second);
     return ret;
